// animation.cc

#include <string>
#include <fstream>
#include <sstream>

#define GLEW_STATIC 1
#include <GL/glew.h>
#include <GL/glut.h>

#include "animation.hpp"
#include <v3.hpp>
#include <matrix3.hpp>

void
Animation::load (const std::string &filename)
{
  std::string   line;
  std::ifstream file (filename.c_str());

  if(!file.good())
  {throw std::string("cannot open file in Animation::load("+filename+")");}

  bones.clear ();
  do
    {
      std::getline (file, line);
    } while (line[0] == '#');
  std::stringstream tokens (line);
  unsigned num_frames, num_bones;
  tokens >> num_frames >> num_bones;

  while (bones.size () != num_frames)
    {
      std::vector<Joint> frame;
      while (frame.size () != num_bones)
        {
          Joint joint;
          std::getline (file, line);
          if (line[0] == '#') continue;
          std::stringstream tokens2 (line);
          joint.parent = 0;
          tokens2 >> joint.parent;
          tokens2 >> joint.position.x() >> joint.position.y() >> joint.position.z();

          double qx=0,qy=0,qz=0;
          tokens2 >> qx >> qy >> qz;
          joint.orientation=cpe::quaternion(cpe::v4(qx,qy,qz,0));
          joint.orientation.compute_w ();

          if (joint.parent >= 0)
            {
              Joint parent_joint = frame[joint.parent];
              joint.position = parent_joint.orientation*joint.position + parent_joint.position;
              joint.orientation = parent_joint.orientation * joint.orientation;
              joint.orientation/=joint.orientation.norm();
            }
          frame.push_back (joint);
        }
      bones.push_back (frame);
    }
}

